@@ -5,8 +5,7 @@ import sys
 
 from spectra_lexer import Spectra, SpectraOptions
 from spectra_lexer.gui_engine import GUIEngine
-from spectra_lexer.gui_ext import GUIExtension
-from spectra_lexer.gui_rest import RESTDisplay, RESTDisplayPage, RESTGUIApplication, RESTUpdate
+from spectra_lexer.gui_rest import RESTGUIApplication
 from spectra_lexer.http.connect import HTTPConnectionHandler
 from spectra_lexer.http.json import CustomJSONEncoder, JSONBinaryWrapper, RestrictedJSONDecoder
 from spectra_lexer.http.service import HTTPDataService, HTTPFileService, HTTPGzipFilter, \
@@ -14,21 +13,19 @@ from spectra_lexer.http.service import HTTPDataService, HTTPFileService, HTTPGzi
 from spectra_lexer.http.tcp import ThreadedTCPServer
 
 HTTP_PUBLIC_DEFAULT = os.path.join(os.path.split(__file__)[0], "http_public")
-JSON_DATA_CLASSES = [RESTDisplay, RESTDisplayPage, RESTUpdate]
 
 
 def build_app(spectra:Spectra) -> RESTGUIApplication:
     """ Start with standard command-line options and build the app. """
+    io = spectra.resource_io
     search_engine = spectra.search_engine
     analyzer = spectra.analyzer
     graph_engine = spectra.graph_engine
     board_engine = spectra.board_engine
-    gui_engine = GUIEngine(search_engine, analyzer, graph_engine, board_engine)
-    io = spectra.resource_io
     translations_paths = spectra.translations_paths
     index_path = spectra.index_path
-    gui_ext = GUIExtension(io, search_engine, analyzer, translations_paths, index_path)
-    gui_ext.load_initial()
+    gui_engine = GUIEngine(io, search_engine, analyzer, graph_engine, board_engine, translations_paths, index_path)
+    gui_engine.load_initial()
     return RESTGUIApplication(gui_engine)
 
 
@@ -36,8 +33,6 @@ def build_dispatcher(app:RESTGUIApplication, root_dir:str, *args) -> HTTPConnect
     """ Build an HTTP server object customized to Spectra's requirements. """
     decoder = RestrictedJSONDecoder(size_limit=100000, obj_limit=20, arr_limit=20)
     encoder = CustomJSONEncoder()
-    for cls in JSON_DATA_CLASSES:
-        encoder.add_data_class(cls)
     json_wrapper = JSONBinaryWrapper(app.run, decoder=decoder, encoder=encoder)
     data_service = HTTPDataService(json_wrapper, "application/json")
     compressed_service = HTTPGzipFilter(data_service, size_threshold=1000)
