@@ -1,8 +1,7 @@
 from typing import Callable, Sequence
 
 from spectra_lexer.console.qt import ConsoleDialog
-from spectra_lexer.gui_engine import GUIEngine, GUIOptions, QueryResults
-from spectra_lexer.gui_ext import GUIExtension
+from spectra_lexer.gui_engine import GUIEngine, GUIOptions
 from spectra_lexer.objtree.main import NamespaceTreeDialog
 from spectra_lexer.qt.board import BoardPanel
 from spectra_lexer.qt.config import ConfigDialog
@@ -15,6 +14,7 @@ from spectra_lexer.qt.system import QtTaskExecutor
 from spectra_lexer.qt.title import TitleDisplay
 from spectra_lexer.qt.window import WindowController
 from spectra_lexer.spc_lexer import TranslationFilter
+from spectra_lexer.util.config import SimpleConfigDict
 
 
 class QtGUIApplication:
@@ -27,11 +27,11 @@ class QtGUIApplication:
 
     last_exception: BaseException = None  # Most recently trapped exception, saved for debug tools.
 
-    def __init__(self, engine:GUIEngine, ext:GUIExtension, tasks:QtTaskExecutor, dialogs:DialogManager,
-                 window:WindowController, menu:MenuController, title:TitleDisplay,
-                 search:SearchPanel, graph:GraphPanel, board:BoardPanel) -> None:
+    def __init__(self, engine:GUIEngine, config:SimpleConfigDict, tasks:QtTaskExecutor, dialogs:DialogManager,
+                 window:WindowController, menu:MenuController,
+                 title:TitleDisplay, search:SearchPanel, graph:GraphPanel, board:BoardPanel) -> None:
         self._engine = engine
-        self._ext = ext
+        self._config = config
         self._tasks = tasks
         self._dialogs = dialogs
         self._window = window
@@ -41,7 +41,6 @@ class QtGUIApplication:
         self._graph = graph
         self._board = board
         self._last_graph_ref = None
-        self._last_example_id = ""
 
     def _set_title(self, text:str) -> None:
         self._title.set_static_text(text)
@@ -51,15 +50,16 @@ class QtGUIApplication:
         self._board.set_caption(caption)
 
     def _set_translation(self, keys:str, letters:str) -> None:
-        """ Format a translation and show it in the search boxes and/or title bar. """
-        self._search.select_translation(keys, letters)
+        """ Format a translation and show it in the title bar. """
         tr_text = " ".join([keys, self.TR_DELIMITER, letters])
         self._set_title(tr_text)
 
-    def _set_example_id(self, example_id:str) -> None:
-        """ Save the current example ID and show/hide the link based on its contents. """
-        self._last_example_id = example_id
-        if example_id:
+    def _set_board(self, board:str) -> None:
+        self._board.set_data(board)
+
+    def _set_link_visible(self, visible:bool) -> None:
+        """ Show/hide the link. """
+        if visible:
             self._board.show_examples_link()
         else:
             self._board.hide_examples_link()
@@ -72,81 +72,59 @@ class QtGUIApplication:
         caption = self._engine.get_caption(ref)
         self._set_caption(caption)
         board = self._engine.draw_board(ref)
-        self._board.set_data(board)
+        self._set_board(board)
         example_id = self._engine.get_example_id(ref)
-        self._set_example_id(example_id)
+        self._set_link_visible(bool(example_id))
 
-    def _clear_page(self) -> None:
-        """ Clear the current display data. """
-        self._last_graph_ref = None
-        self._last_example_id = ""
-        self._graph.set_plaintext("")
-        self._set_caption("")
-        self._board.set_data("")
-        self._set_example_id("")
-
-    def _find_same_example(self, refs:Sequence[str]):
-        """ Attempt to find a graph ref that has the same link target as the last selection. """
-        if self._last_example_id:
-            for ref in refs:
-                if self._last_example_id == self._engine.get_example_id(ref):
-                    return ref
-        return ""
-
-    def _update_display(self, results:QueryResults) -> None:
-        """ Update the GUI display with a new analysis.
-            Attempt to show a page using the last link target, otherwise show the default.
-            Forcibly reset the graph's focus before setting the start page. """
-        self._set_translation(results.keys, results.letters)
-        start_ref = self._find_same_example(results.refs)
+    def _show_start_page(self, start_ref:str) -> None:
+        """ Forcibly reset the graph's focus before setting the start page. """
         intense = bool(start_ref)
         self._graph.set_focus(intense)
         self._show_page(start_ref, intense)
 
-    def _get_config(self) -> dict:
-        return self._ext.get_config()
+    def _clear_page(self) -> None:
+        """ Clear the current display data. """
+        self._last_graph_ref = None
+        self._graph.set_plaintext("")
+        self._set_caption("")
+        self._set_board("")
+        self._set_link_visible(False)
 
-    def _update_config(self, options:dict) -> None:
-        self._ext.update_config(options)
-
-    def _base_options(self) -> GUIOptions:
-        """ Config settings form the base for a set of GUI engine options. """
-        return GUIOptions(self._get_config())
+    def _gui_options(self) -> dict:
+        """ Return all engine options derived from GUI controls. """
+        return {"search_mode_strokes": self._search.is_mode_strokes(),
+                "search_mode_regex":   self._search.is_mode_regex(),
+                "board_aspect_ratio":  self._board.aspect_ratio(),
+                "board_show_compound": self._board.is_compound(),
+                "board_show_letters":  self._board.shows_letters()}
 
     def set_options(self, **kwargs) -> None:
-        """ Set all options that may be needed by the steno engine. Overriding options may be given as kwargs. """
-        opts = self._base_options()
-        opts.search_mode_strokes = self._search.is_mode_strokes()
-        opts.search_mode_regex = self._search.is_mode_regex()
-        opts.board_aspect_ratio = self._board.aspect_ratio()
-        opts.board_show_compound = self._board.is_compound()
-        opts.board_show_letters = self._board.shows_letters()
-        if kwargs:
-            vars(opts).update(kwargs)
-        self._engine.set_options(opts)
-
-    def set_translations(self, *args) -> None:
-        self._ext.set_translations(*args)
+        """ Set all options that may be needed by the steno engine from various sources.
+            Source priority is: config file < GUI controls < keyword arguments. """
+        options = {**self._config, **self._gui_options(), **kwargs}
+        self._engine.set_options(options)
 
     def gui_search(self, pattern:str, pages:int) -> None:
         """ Run a translation search and update the GUI with any results. """
-        search_results = self._engine.search(pattern, pages)
-        self._search.update_results(search_results)
+        matches = self._engine.search(pattern, pages)
+        self._search.update_results(matches)
 
-    def gui_query(self, keys_or_seq:Sequence[str], letters:str) -> None:
-        """ Run a lexer query and update the GUI with any results. """
-        query_results = self._engine.query(keys_or_seq, letters)
-        self._update_display(query_results)
-
-    def gui_search_examples(self) -> None:
-        """ Run an example search based on the last valid link reference and update the GUI with any results. """
-        pattern = self._engine.random_pattern(self._last_example_id)
-        if pattern:
-            self._search.update_input(pattern)
-            search_results = self._engine.search(pattern)
-            self._search.update_results(search_results)
-            query_results = self._engine.query_random(search_results)
-            self._update_display(query_results)
+    def gui_query(self, keys:str, letters:str) -> None:
+        """ Run a lexer query and update the GUI with the new analysis.
+            Attempt to show a page using the last link target, otherwise show the default.
+            Forcibly reset the graph's focus before setting the start page. """
+        match, mapping = self._engine.search_selection(keys, letters)
+        self._search.select(match, mapping)
+        self._set_translation(keys, letters)
+        last_id = self._engine.get_example_id(self._last_graph_ref or "")
+        refs = self._engine.query(keys, letters)
+        start_ref = ""
+        if last_id:
+            for ref in refs:
+                if last_id == self._engine.get_example_id(ref):
+                    start_ref = ref
+                    break
+        self._show_start_page(start_ref)
 
     def _on_translation_edit(self, _:str) -> None:
         """ Display user entry instructions in the caption. """
@@ -166,6 +144,28 @@ class QtGUIApplication:
         self.set_options()
         self.gui_query(keys, letters)
 
+    def _on_search_input(self, pattern:str, pages:int) -> None:
+        self.set_options()
+        self.gui_search(pattern, pages)
+
+    def _on_search_query(self, match:str, mappings:Sequence[str]) -> None:
+        self.set_options()
+        keys, letters = self._engine.best_translation(match, mappings)
+        self.gui_query(keys, letters)
+
+    def _on_request_examples(self) -> None:
+        """ Run an example search and update the GUI with any results. """
+        if self._last_graph_ref is not None:
+            self.set_options()
+            example_id = self._engine.get_example_id(self._last_graph_ref)
+            pattern = self._engine.random_pattern(example_id)
+            if pattern:
+                self._search.update_input(pattern)
+                results = self._engine.search(pattern)
+                self._search.update_results(results)
+                keys, letters = self._engine.random_translation(results)
+                self.gui_query(keys, letters)
+
     def _on_graph_action(self, node_ref:str, intense:bool) -> None:
         """ On mouse actions, change the current analysis page to the one under <node_ref>. """
         self._show_page(node_ref, intense)
@@ -175,7 +175,7 @@ class QtGUIApplication:
         if self._last_graph_ref is not None:
             self.set_options()
             board = self._engine.draw_board(self._last_graph_ref)
-            self._board.set_data(board)
+            self._set_board(board)
 
     def _on_board_save(self) -> None:
         """ Save the current board diagram on link click. """
@@ -183,18 +183,6 @@ class QtGUIApplication:
         if filename:
             self._board.dump_image(filename)
 
-    def _on_search_input(self, pattern:str, pages:int) -> None:
-        self.set_options()
-        self.gui_search(pattern, pages)
-
-    def _on_search_query(self, keys_or_seq:Sequence[str], letters:str) -> None:
-        self.set_options()
-        self.gui_query(keys_or_seq, letters)
-
-    def _on_request_examples(self) -> None:
-        self.set_options()
-        self.gui_search_examples()
-
     def _connect_signals(self) -> None:
         self._title.connect_signals(self._on_translation_edit, self._on_translation_submit)
         self._search.connect_signals(self._on_search_input, self._on_search_query)
@@ -256,12 +244,15 @@ class QtGUIApplication:
     def async_finish(self, msg_finish:str) -> None:
         self.async_queue(self._unblock, msg_finish)
 
+    def set_translations(self, *args) -> None:
+        self._engine.set_translations(*args)
+
     def open_translations(self) -> None:
         """ Present a dialog for the user to select translation files and attempt to load them all unless cancelled. """
         filenames = self._dialogs.open_files("Load Translations", "json")
         if filenames:
             self.async_start("Loading files...")
-            self.async_run(self._ext.load_translations, *filenames)
+            self.async_run(self._engine.load_translations, *filenames)
             self.async_finish("Loaded translations from file dialog.")
 
     def open_index(self) -> None:
@@ -269,20 +260,41 @@ class QtGUIApplication:
         filename = self._dialogs.open_file("Load Index", "json")
         if filename:
             self.async_start("Loading file...")
-            self.async_run(self._ext.load_examples, filename)
+            self.async_run(self._engine.load_examples, filename)
             self.async_finish("Loaded index from file dialog.")
 
+    # Info for engine options which should be saved in a CFG file.
+    CONFIG_INFO = {
+        "search_match_limit": ["Match Limit", "Maximum number of matches returned on one page of a search."],
+        "lexer_strict_mode": ["Strict Mode", "Only return lexer results that match every key in a translation."],
+        "graph_compressed_layout": ["Compressed Layout", "Compress the graph layout vertically to save space."]
+    }
+
+    def _config_load(self) -> None:
+        """ Load config options from disk. """
+        self._config.read()
+
+    def _config_update(self, options:dict) -> None:
+        """ Update the config options and save them to disk. """
+        self._config.update(options)
+        self._config.write()
+
+    def _config_set_defaults(self) -> None:
+        """ Update the config options with default values used by the engine. """
+        defaults = {key: getattr(GUIOptions, key) for key in self.CONFIG_INFO}
+        self._config_update(defaults)
+
     def _config_edited(self, options:dict) -> None:
-        self._update_config(options)
+        """ Update the config options with values from the user dialog. """
+        self._config_update(options)
         self.set_status("Configuration saved.")
 
     def config_editor(self) -> None:
         """ Create and show the GUI configuration manager dialog with info from all active components. """
         dialog = self._dialogs.load(ConfigDialog)
         if dialog is not None:
-            opts = self._base_options()
-            for key, name, description in opts.CFG_OPTIONS:
-                value = getattr(opts, key)
+            for key, (name, description) in self.CONFIG_INFO.items():
+                value = self._config.get(key)
                 dialog.add_option(key, value, "General", name, description)
                 dialog.call_on_options_accept(self._config_edited)
             dialog.show()
@@ -290,7 +302,7 @@ class QtGUIApplication:
     def _make_index(self, size:int) -> None:
         """ Make a custom-sized index. Disable the GUI while processing and show a success message when done. """
         self.async_start("Making new index...")
-        self.async_run(self._ext.compile_examples, size)
+        self.async_run(self._engine.compile_examples, size)
         self.async_finish("Successfully created index!")
 
     def confirm_startup_index(self) -> None:
@@ -326,15 +338,15 @@ class QtGUIApplication:
             If the config settings are blank, this is the first time the program has been run.
             Add the default config values and present an index generation dialog in this case. """
         self._connect_signals()
-        if not self._get_config():
-            defaults = {key: getattr(GUIOptions, key) for key, *_ in GUIOptions.CFG_OPTIONS}
-            self._update_config(defaults)
+        self._config_load()
+        if not self._config:
+            self._config_set_defaults()
             self.confirm_startup_index()
 
     def start(self) -> None:
         """ Show the window and load initial data asynchronously on a new thread to keep the GUI responsive. """
         self._block("Loading...")
-        self.async_run(self._ext.load_initial)
+        self.async_run(self._engine.load_initial)
         self.async_finish("Loading complete.")
         self.async_queue(self._on_ready)
         self.show()
