@@ -6,7 +6,7 @@ from spectra_lexer.graph.body import BoldBody, SeparatorBody, ShiftedBody, Stand
 from spectra_lexer.graph.canvas import GridCanvas
 from spectra_lexer.graph.connectors import InversionConnectors, LinkedConnectors, NullConnectors, \
                                            SimpleConnectors, ThickConnectors, UnmatchedConnectors
-from spectra_lexer.graph.format import HTMLFormatter
+from spectra_lexer.graph.format import HTML_COMPAT, HTML_STANDARD, HTMLFormatter
 from spectra_lexer.graph.layout import CascadedGraphLayout, CompressedGraphLayout, GraphLayout, LayoutParams
 from spectra_lexer.resource.rules import StenoRule
 
@@ -132,6 +132,20 @@ class GraphTree(RuleMapping):
         """ Use the root node as a default. """
         return self._tree_map.get(k, self._root_rule)
 
+    def caption(self, ref="") -> str:
+        """ Generate a caption for the node at <ref>. The root node shows its info only. Others also show the keys.
+            Nodes with children show the complete mapping of keys to letters. """
+        rule = self[ref]
+        keys = rule.keys
+        letters = rule.letters
+        info = rule.info
+        if rule is self._root_rule:
+            return info
+        elif rule.rulemap and letters:
+            return f'{keys} → {letters}: {info}'
+        else:
+            return f'{keys}: {info}'
+
     def draw(self, ref="", *, intense=False) -> HTMLGraph:
         """ Return an HTML text graph with <ref> highlighted.
             Highlight nothing if <ref> is blank. Use brighter highlighting colors if <intense> is True. """
@@ -148,7 +162,7 @@ class GraphEngine:
     def _build_body(self, rule:StenoRule) -> IBody:
         """ Make a node display body. The text is shifted left if it starts with an ignored token. """
         keys = rule.keys
-        if rule:
+        if rule.rulemap:
             body = BoldBody(rule.letters)
         elif keys == self._key_sep:
             body = SeparatorBody(keys)
@@ -163,10 +177,10 @@ class GraphEngine:
         if rule.is_inversion:
             # Node for an inversion of steno order. Connectors should indicate some kind of "reversal".
             connectors = InversionConnectors(length, width)
-        elif rule.is_linked:
+        elif rule.is_split:
             # Node for a child rule that uses keys from two strokes. This complicates stroke delimiting.
             connectors = LinkedConnectors(length, width)
-        elif rule:
+        elif rule.rulemap:
             connectors = ThickConnectors(length, width)
         elif rule.keys == self._key_sep:
             connectors = NullConnectors()
@@ -180,7 +194,7 @@ class GraphEngine:
         """ Build a display node tree recursively from a rule's properties, position, and descendants. """
         ref = str(len(tree_map))
         tree_map[ref] = rule
-        children = [self._build_tree(tree_map, c.child, c.start, c.length) for c in rule]
+        children = [self._build_tree(tree_map, item.child, item.start, item.length) for item in rule.rulemap]
         body = self._build_body(rule)
         width = body.width()
         connectors = self._build_connectors(rule, length, width)
@@ -196,5 +210,6 @@ class GraphEngine:
         canvas = TextCanvas(EMPTY_ELEMENT)
         root.draw(canvas)
         grid = canvas.to_lists()
-        formatter = HTMLFormatter(grid, compat=compat)
+        fmt = HTML_COMPAT if compat else HTML_STANDARD
+        formatter = HTMLFormatter(grid, fmt)
         return GraphTree(rule, tree_map, formatter)
