@@ -1,11 +1,10 @@
 from collections import defaultdict
 from typing import Iterable, List, Mapping
 
-from spectra_lexer.lexer import LexerRule
-from spectra_lexer.lexer.lexer import LexerResult, StenoLexer
+from spectra_lexer.lexer.lexer import LexerResult, LexerRule, StenoLexer
 from spectra_lexer.lexer.parallel import ParallelMapper
 from spectra_lexer.resource.keys import StenoKeyConverter
-from spectra_lexer.resource.rules import StenoRule
+from spectra_lexer.resource.rules import StenoRule, StenoRuleFactory
 from spectra_lexer.resource.translations import ExamplesDict, RuleID, Translation
 
 TranslationPairs = Iterable[Translation]  # Iterable collection of steno translations.
@@ -44,58 +43,48 @@ class TranslationFilter:
 class StenoAnalyzer:
     """ Key-converting wrapper for the lexer. Also uses multiprocessing to make an examples index. """
 
-    def __init__(self, to_skeys:StenoKeyConverter, to_rtfcre:StenoKeyConverter, lexer:StenoLexer,
-                 rule_sep:StenoRule, refmap:Mapping[LexerRule, StenoRule], idmap:Mapping[LexerRule, RuleID]) -> None:
-        self._to_skeys = to_skeys    # Converts user RTFCRE steno strings to s-keys.
-        self._to_rtfcre = to_rtfcre  # Converts s-keys back to RTFCRE.
+    # Rule info strings for analysis results. The output is nowhere near reliable if some keys are unmatched.
+    INFO_COMPLETE = "Found complete match."
+    INFO_INCOMPLETE = "Incomplete match. Not reliable."
+    INFO_EMPTY = "No matches found."
+
+    def __init__(self, converter:StenoKeyConverter, lexer:StenoLexer, factory:StenoRuleFactory,
+                 refmap:Mapping[LexerRule, StenoRule], idmap:Mapping[LexerRule, RuleID]) -> None:
+        self._converter = converter  # Converts between RTFCRE and s-keys formats.
         self._lexer = lexer          # Main analysis engine; operates only on s-keys.
-        self._rule_sep = rule_sep    # Stroke separator rule.
+        self._factory = factory      # Creates steno rules from analysis data.
         self._refmap = refmap        # Mapping of lexer rule objects to their original StenoRules.
         self._idmap = idmap          # Mapping of lexer rule objects to valid example rule IDs.
 
-    @staticmethod
-    def _result_info(result:LexerResult) -> str:
-        """ Return an info string for this result. The output is nowhere near reliable if some keys are unmatched. """
-        if not result.unmatched_skeys:
-            info = "Found complete match."
-        elif result.rules:
-            info = "Incomplete match. Not reliable."
-        else:
-            info = "No matches found."
-        return info
+    def _to_skeys(self, keys:str) -> str:
+        """ Convert user RTFCRE steno <keys> to s-keys. """
+        return self._converter.rtfcre_to_skeys(keys)
+
+    def _to_rtfcre(self, skeys:str) -> str:
+        """ Convert <skeys> back to RTFCRE format. """
+        return self._converter.skeys_to_rtfcre(skeys)
 
     def query(self, keys:str, letters:str, *, strict_mode=False) -> StenoRule:
         """ Return a lexer analysis matching <keys> to <letters> in standard steno rule format.
             If <strict_mode> is True and the best result is missing keys, return a fully unmatched result instead. """
         skeys = self._to_skeys(keys)
         result = self._lexer.query(skeys, letters)
-        keys = self._to_rtfcre(skeys)
-        info = self._result_info(result)
-        rule = StenoRule(keys, letters, info)
+        self._factory.push()
+        if strict_mode and result.unmatched_skeys:
+            result = LexerResult([], [], skeys)
+        for lr, start in zip(result.rules, result.rule_positions):
+            child = self._refmap[lr]
+            length = len(lr.letters)
+            self._factory.connect(child, start, length)
+        info = self.INFO_COMPLETE
         unmatched_skeys = result.unmatched_skeys
-        if strict_mode and unmatched_skeys:
-            unmatched_skeys = skeys
-        else:
-            for lr, start in zip(result.rules, result.rule_positions):
-                child = self._refmap[lr]
-                length = len(lr.letters)
-                rule.add_connection(child, start, length)
         if unmatched_skeys:
+            info = self.INFO_INCOMPLETE if result.rules else self.INFO_EMPTY
             unmatched_keys = self._to_rtfcre(unmatched_skeys)
-            rule.add_unmatched(unmatched_keys)
-        return rule
-
-    def compound_query(self, translations:TranslationPairs, **kwargs) -> StenoRule:
-        """ Return a compound lexer analysis of several translations joined together. """
-        rules = [self.query(keys, letters, **kwargs) for keys, letters in translations]
-        n = len(rules)
-        if not n:
-            raise ValueError("Need at least 1 translation, got 0.")
-        if n == 1:
-            return rules[0]
-        delimited_seq = [self._rule_sep] * (2 * n - 1)
-        delimited_seq[::2] = rules
-        return StenoRule.join(delimited_seq)
+            nletters = len(letters)
+            self._factory.connect_unmatched(unmatched_keys, nletters, "unmatched keys")
+        keys = self._to_rtfcre(skeys)
+        return self._factory.build(keys, letters, info)
 
     def best_translation(self, keys_iter:Iterable[str], letters:str) -> str:
         """ Return the best (most accurate) match to <letters> out of <keys_iter> according to lexer ranking. """
@@ -137,6 +126,29 @@ class StenoAnalyzer:
                 index[r_id][keys] = letters
         return index
 
+    def delimit(self, rules:Iterable[StenoRule], keys_delim:str, letters_delim="") -> List[StenoRule]:
+        """ Delimit a sequence of rules with <keys_delim> and <letters_delim>. """
+        self._factory.push()
+        rule_delim = self._factory.build(keys_delim, letters_delim, "delimiter")
+        delimited = []
+        for r in rules:
+            delimited += [r, rule_delim]
+        return delimited[:-1]
+
+    def join(self, rules:Iterable[StenoRule]) -> StenoRule:
+        """ Join several rules into one by concatenation. """
+        skeys = letters = ""
+        offset = 0
+        self._factory.push()
+        for r in rules:
+            skeys += self._to_skeys(r.keys)
+            letters += r.letters
+            length = len(r.letters)
+            self._factory.connect(r, offset, length)
+            offset += length
+        keys = self._to_rtfcre(skeys)
+        return self._factory.build(keys, letters, "combined rules")
+
     def normalize_keys(self, keys:str) -> str:
         """ Normalize a set of RTFCRE keys by converting back and forth. """
         skeys = self._to_skeys(keys)
